import FreezeTool
import sys, os

############### CUSTOMIZE THESE ITEMS #######################################
# Thirdparty libraries
THIRDPARTY_DIR = "/Users/davidmarques/prog/panda3d-webgl-port/thirdparty/emscripten-libs"
# source of Emscripten-compiled python
PY_MODULE_DIR = "/Users/davidmarques/prog/Python-3.8.6/Modules"
PY_STDLIB_DIR = "/Users/davidmarques/prog/Python-3.8.6/lib"
# source of Emscripten-compiled panda3d
PANDA_BUILT_DIR = "/Users/davidmarques/prog/panda3d-webgl-port/embuilt"
# Files to preload into the virtual filesystem
# PRELOAD_FILES = []
PRELOAD_FILES = ["models/.DS_Store","models/sphere.bam","models/door.bam","models/floor.bam","models/wall.bam","pics/exit_icon.png","pics/grey_box.png","pics/teleport.png","pics/.DS_Store","pics/grey_star.png","pics/help_icon.png","pics/map_right.png","pics/trolly_icon.png","pics/map_down.png","pics/restart.png","pics/hand_icon.png","pics/door_icon.png","pics/gold_box.png","pics/wasd_icon.png","pics/gold_star.png","pics/cube_icon.png","pics/arrows_icon.png","pics/blank_wall.png","pics/map_left.png","pics/mouse_icon.png","pics/turn_icon.png","pics/descr_icon.png","pics/menu_back.png","pics/map_up.png","pics/menu_back.jpg","pics/trolly_icon_full.png","pics/pmove_icon.png","pics/rooms_grid.png","audio/.DS_Store","audio/correct-answer-2.ogg","audio/incorrect-answer.ogg","audio/bell-ring-01.ogg","audio/cheering.ogg","images/Stonehenge.jpg","images/Hammurabi.jpg","images/Screen_Shot_2013-03-16_at_11.10.31_AM.jpg","images/.DS_Store","images/JAWS_Movie_poster.jpg","images/Chichen_Itza.jpg","images/Screen_Shot_2013-03-16_at_12.05.11_PM.jpg","images/Mesozoic_02.jpg","images/Screen_Shot_2013-03-16_at_11.08.52_AM.png","images/Deep_sea_life.jpg","images/Pleistocene_02.jpg","images/cambrian_diorama_opener.jpg","images/wild_honeybee.jpg","images/Screen_Shot_2013-03-16_at_12.43.50_PM.jpg","images/amb_full_small.jpg","images/virus_generic.jpg","images/Screen_Shot_2013-03-16_at_12.51.46_PM.jpg","floors/PavingStones060_2K_Roughness.jpg","floors/Steel_C.jpg","floors/PavingStones060_2K_AmbientOcclusion.jpg","floors/.DS_Store","floors/PavingStones060_2K_Color.jpg","floors/PavingStones060_2K_Displacement.jpg","floors/PavingStones060_2K_Normal.jpg"]
# must be a multiple of 2**16
MEMNEEDED = 2000 * (2**16)
APPNAME = "museum"
##############################################################################

print("total memory: "+str(MEMNEEDED))

# Python extension modules
# PY_MODULES = ["getbuildinfo", "xxsubtype", "symtablemodule", "hashtable", "_tracemalloc", "faulthandler", "stringio", "textio", "bufferedio", "bytesio", "fileio", "iobase", "_iomodule", "_localemodule", "_threadmodule", "timemodule", "_stat", "signalmodule", "atexitmodule", "itertoolsmodule", "_abc", "_collectionsmodule", "_operator", "_functoolsmodule", "_weakref", "_codecsmodule", "_sre", "pwdmodule", "errnomodule", "posixmodule", "gcmodule", "main", "getpath", "config"]

# Panda modules / libraries
PANDA_MODULES = ["core", "direct", "physics"]
PANDA_LIBS = ["libp3pandatoolbase", "libp3pstatserver", "libp3xfileegg", "libp3xfile", "libp3vrmlegg", "libp3vrml", "libp3lwoegg", "libp3lwo", "libp3imagebase", "libp3fltegg", "libp3flt", "libp3eggcharbase", "libp3palettizer", "libp3objegg", "libp3dxfegg", "libp3dxf", "libp3eggbase", "libp3progbase", "libp3converter"]
PANDA_MODS =["libp3direct", "libpandaphysics", "libpandabullet", "libp3webgldisplay", "libpandaegg", "libp3framework", "libp3openal_audio", "libpandafx", "libpandaskel", "libp3vision", "libpanda", "libpandaexpress", "libp3interrogatedb", "libp3dtoolconfig", "libp3dtool", "libp3ptloader", "libpandaai"]


class EmscriptenEnvironment:
    platform = 'emscripten'

    pythonInc = THIRDPARTY_DIR + "/python/include/python3.8"
    pythonLib = THIRDPARTY_DIR + "/python/lib/libpython3.8.a "# + THIRDPARTY_DIR + "/openssl/lib/libssl.a " + THIRDPARTY_DIR + "/openssl/lib/libcrypto.a"

    modStr = " "#.join((os.path.join(PY_MODULE_DIR, a + ".o") for a in PY_MODULES))

    pandaFlags = ""
    for mod in PANDA_MODULES:
        pandaFlags += " %s/panda3d/%s.so" % (PANDA_BUILT_DIR, mod)

    # for lib in PANDA_LIBS:
    #     pandaFlags += " %s/lib/%s.a" % (PANDA_BUILT_DIR, lib)

    # for lib in PANDA_MODS:
    #     pandaFlags += " %s/lib/%s.o" % (PANDA_BUILT_DIR, lib)

    for subdir, dirs, files in os.walk(PANDA_BUILT_DIR+"/lib"):
        for file in files:
            pandaFlags+= ' '+os.path.join(subdir, file)


    pandaFlags += " -I%s/include" % (PANDA_BUILT_DIR)
    pandaFlags += " -s USE_ZLIB=1 -s USE_VORBIS=1 -s USE_LIBPNG=1 -s USE_FREETYPE=1 -s USE_HARFBUZZ=1 -s ERROR_ON_UNDEFINED_SYMBOLS=0 "
    pandaFlags += " -s DISABLE_EXCEPTION_THROWING=0 "
    pandaFlags += " -s 'EXTRA_EXPORTED_RUNTIME_METHODS=[\"cwrap\"]'"

    for file in PRELOAD_FILES:
        pandaFlags += " --preload-file %s" % file

    compileObj = "emcc -O3 -s ASSERTIONS=0 -s NO_EXIT_RUNTIME=1 -fno-exceptions -fno-rtti -c -o %(basename)s.o %(filename)s -I" + pythonInc
    linkExe = "emcc -O3 -s TOTAL_MEMORY="+str(MEMNEEDED)+" -s ASSERTIONS=0 -s MAX_WEBGL_VERSION=2 -s NO_EXIT_RUNTIME=1 -fno-exceptions -fno-rtti -o %(basename)s.html %(basename)s.o  " + modStr + " " + pythonLib + " " + pandaFlags
    linkDll = "emcc -O2 -shared -o %(basename)s.o %(basename)s.o " + pythonLib

    # Paths to Python stuff.
    Python = None
    PythonIPath = pythonInc
    PythonVersion = "3.8"

    suffix64 = ''
    dllext = ''
    arch = ''

    def compileExe(self, filename, basename, extraLink=[]):
        compile = self.compileObj % {
            'python' : self.Python,
            'filename' : filename,
            'basename' : basename,
            }
        print ("Now doing: "+ compile)
        if os.system(compile) != 0:
            raise ( 'failed to compile %s.' % basename)

        link = self.linkExe % {
            'python' : self.Python,
            'filename' : filename,
            'basename' : basename,
            }
        link += ' ' + ' '.join(extraLink)
        print ("Now doing: "+ link)
        if os.system(link) != 0:
            raise ('failed to link %s.' % basename)

    def compileDll(self, filename, basename, extraLink=[]):
        compile = self.compileObj % {
            'python' : self.Python,
            'filename' : filename,
            'basename' : basename,
            }
        print ("Now doing: "+ compile)
        if os.system(compile) != 0:
            raise ('failed to compile %s.' % basename)

        link = self.linkDll % {
            'python' : self.Python,
            'filename' : filename,
            'basename' : basename,
            'dllext' : self.dllext,
            }
        link += ' ' + ' '.join(extraLink)
        print ("Now doing: "+ link)
        if os.system(link) != 0:
            raise ('failed to link %s.' % basename)


freezer = FreezeTool.Freezer()
freezer.frozenMainCode = """
#include "Python.h"
#include <emscripten.h>

extern PyObject *PyInit_core();
extern PyObject *PyInit_direct();
extern PyObject *PyInit_physics();

extern void init_libOpenALAudio();
extern void init_libwebgldisplay();
extern void init_libpnmimagetypes();

extern void task_manager_poll();


EMSCRIPTEN_KEEPALIVE void loadPython() {
    Py_VerboseFlag = 1;
    Py_FrozenFlag = 1; /* Suppress errors from getpath.c */
    Py_DontWriteBytecodeFlag = 1;
    Py_NoSiteFlag = 1;
    Py_NoUserSiteDirectory = 1;

    // Py_SetProgramName("emscripthon");
    Py_InitializeEx(0);

    fprintf(stderr, "Python %s\\n",
        Py_GetVersion());

    //PySys_SetArgv(argc, argv);

    EM_ASM({
        Module.setStatus('Importing Panda3D...');
        window.setTimeout(_loadPanda, 0);
    });
}


EMSCRIPTEN_KEEPALIVE void loadPanda() {
    PyObject *panda3d_module = PyImport_AddModule("panda3d");
    PyModule_AddStringConstant(panda3d_module, "__package__", "panda3d");
    PyModule_AddObject(panda3d_module, "__path__", PyList_New(0));

    PyObject *panda3d_dict = PyModule_GetDict(panda3d_module);

    PyObject *core_module = PyInit_core();
    PyDict_SetItemString(panda3d_dict, "core", core_module);

    PyObject *direct_module = PyInit_direct();
    PyDict_SetItemString(panda3d_dict, "direct", direct_module);

    PyObject *physics_module = PyInit_physics();
    PyDict_SetItemString(panda3d_dict, "physics", physics_module);

    PyObject *sys_modules = PySys_GetObject("modules");
    PyDict_SetItemString(sys_modules, "panda3d.core", core_module);
    PyDict_SetItemString(sys_modules, "panda3d.direct", direct_module);

    init_libOpenALAudio();
    init_libpnmimagetypes();
    init_libwebgldisplay();

    EM_ASM({
        Module.setStatus('Done loading Panda3d.');
    });

    emscripten_set_main_loop(&task_manager_poll, 0, 0);
    if (PyImport_ImportFrozenModule("__main__") < 0) {
      PyErr_Print();
    }
    EM_ASM({
        Module.setStatus('Starting Museum...');
    });

}

int
Py_FrozenMain(int argc, char **argv)
{
    EM_ASM({
        Module.setStatus('Starting Python...');
        window.setTimeout(_loadPython, 0);
    });

    return 0;
}

"""

freezer.cenv = EmscriptenEnvironment()
freezer.excludeModule('doctest')
freezer.excludeModule('difflib')
freezer.addModule('__main__', filename="main.py")

freezer.moduleSearchPath = [PANDA_BUILT_DIR, PY_STDLIB_DIR, PY_MODULE_DIR]

freezer.done(addStartupModules=True)
freezer.generateCode(APPNAME, compileToExe=True)

# for i in freezer.extras:
#     print(i)

# for i in freezer.modules.keys():
#     print(i)
